"""Api testing of pyInels library."""
from pyinels.api import Api

from pyinels.exception import ApiException
from tests.const_test import (
    TEST_API_CLASS_NAMESPACE,
    TEST_API_NAMESPACE,
    TEST_API_ROOM_DEVICES,
    TEST_API_READ_DATA,
    TEST_HOST,
    TEST_PORT,
    TEST_RAW_LIGHT,
    TEST_RAW_DUPLICIT_DEVICES,
    TEST_ROOMS,
    TEST_VERSION
)

from unittest.mock import patch, Mock
from unittest import TestCase

LIGHT_ID = 'SV_12_Garage'
LIGHT_NAME = 'Main light'

LIGHT_RETURN_OFF = {LIGHT_ID: 0}
LIGHT_RETURN_ON = {LIGHT_ID: 1}


class ApiTest(TestCase):
    """Class to test pyInels api library."""

    def setUp(self):
        """Setup all necessary instances nad mocks."""
        self.patches = [
            patch(f'{TEST_API_CLASS_NAMESPACE}.ping', return_value=True),
        ]

        self.api = Api(TEST_HOST, TEST_PORT, TEST_VERSION)

        # patching ping method in IneslBus3. It will be executed every test
        for p in self.patches:
            p.start()

    def tearDown(self):
        """Destroy all instances and mocks."""
        self.api = None
        patch.stopall()
        self.patches = None

    @patch(TEST_API_CLASS_NAMESPACE)
    def test_class_calling(self, mock_class):
        """Class instance test."""
        mock_class(TEST_HOST, TEST_PORT, TEST_VERSION)

        mock_class.assert_called()
        mock_class.assert_called_once()
        mock_class.assert_called_with(TEST_HOST, TEST_PORT, TEST_VERSION)

        self.assertEqual(mock_class.call_count, 1)

    @patch('xmlrpc.client.ServerProxy')
    async def test_connection_failed(self, mock_server):
        """Test api connection."""
        mock_server.return_value = Mock()
        mock_server.side_effect = ApiException(
            'common_exception', 'Exception occur')

        ret = await self.api.ping()
        self.assertEqual(True, ret)
        self.assertEqual(mock_server.call_count, 0)

    @patch(f'{TEST_API_CLASS_NAMESPACE}.ping')
    async def test_ping_failed(self, mock_method):
        """Test ping method."""
        mock_method.return_value = False
        ret = await self.api.ping()

        self.assertEqual(mock_method.call_count, 1)
        self.assertFalse(ret)

    @patch(f'{TEST_API_CLASS_NAMESPACE}.getPlcIp')
    async def test_getPlcIp_success(self, mock_method):
        """Test Ip address of the PLC."""
        RET_VAL = "192.168.2.10"

        mock_method.return_value = RET_VAL
        ret = await self.api.getPlcIp()

        self.assertEqual(mock_method.call_count, 1)
        self.assertEqual(ret, RET_VAL)

    @patch(f'{TEST_API_CLASS_NAMESPACE}.getRooms')
    async def test_getRoomsRaw_list(self, mock_method):
        """Test list of rooms defined on Connection server."""
        mock_method.return_value = TEST_ROOMS

        ret = await self.api.getRooms()

        self.assertEqual(mock_method.call_count, 1)
        self.assertEqual(len(ret), 6)
        self.assertEqual(ret[1], 'First floor')

    @patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_ROOM_DEVICES}')
    async def test_getRoomDevices_list(self, mock_method):
        """Test list of all devices in room."""
        mock_method.return_value = TEST_RAW_LIGHT
        raw = await self.api.getRoomDevicesRaw('room')

        self.assertEqual(mock_method.call_count, 1)
        self.assertEqual(len(raw), len(mock_method.return_value))

        # this is the way how to mock private method
        with patch.object(
            self.api,
            TEST_API_READ_DATA,
            return_value=LIGHT_RETURN_OFF
        ):
            with patch.object(self.api, 'ping', return_value=True):
                obj_list = self.api.getRoomDevices('room')

                device = obj_list[0]

                self.assertGreater(len(obj_list), 0)
                self.assertEqual(device.id, LIGHT_ID)
                self.assertEqual(device.title, LIGHT_NAME)

                patch.stopall()
                device_value = self.api.read([device.id])
                self.assertEqual(device_value, LIGHT_RETURN_OFF)

    @patch(f'{TEST_API_NAMESPACE}.resources.ApiResource.observe')
    async def test_not_duplicit_entries(self, mock_method_observe):
        """Test duplicit entries inside of device list."""
        mock_method_observe.return_value = 0

        with patch.object(self.api, 'ping', return_value=True):
            with patch.object(self.api, TEST_API_ROOM_DEVICES,
                              return_value=TEST_RAW_DUPLICIT_DEVICES):
                obj_list = self.api.getRoomDevices('room')

                with await patch.object(self.api, "getRooms",
                                        return_value=["room"]):

                    with patch.object(self.api, "getRoomDevices",
                                      return_value=obj_list):
                        devices = self.api.getAllDevices()

                        self.assertEqual(len(devices), 48)
                        self.assertGreater(len(obj_list), len(devices))



"""Inels shutter class to control blinds."""
from pyinels.device.pyBase import pyBase
# from pyinels.pyTimer import pyTimer

from pyinels.const import (
    ATTR_DOWN,
    ATTR_UP,
    ATTR_STOP,
    ATTR_SWITCH_ON,
    ATTR_SWITCH_OFF,
    DIRECTIONS_DICT,
    RANGE_BLIND,
    SUPPORT_OPEN,
    SUPPORT_CLOSE,
    SUPPORT_SET_POSITION,
    SUPPORT_STOP,
    SUPPORT_OPEN_TILT,
    SUPPORT_CLOSE_TILT,
    SUPPORT_STOP_TILT,
    STATE_OPEN,
    STATE_CLOSING,
    STATE_OPENING,
    # STATE_CLOSED
)

MIN_RANGE = RANGE_BLIND[0]
MAX_RANGE = RANGE_BLIND[1]


class pyShutter(pyBase):
    """Inels class shutter."""

    async def __init__(self, device):
        """Initialize shutter."""
        await super().__init__(device)
        # self._timer = pyTimer()
        # self.__time_to_stop = 0
        # self.__last_position = MAX_RANGE

    @property
    def state(self):
        """State where the shutter is."""
        up_device = self.up
        down_device = self.down

        up_on = up_device == ATTR_SWITCH_ON \
            and down_device == ATTR_SWITCH_OFF
        down_on = up_device == ATTR_SWITCH_OFF \
            and down_device == ATTR_SWITCH_ON

        # if self.should_stop:
        #     if up_on and not down_on:
        #         state = STATE_OPEN
        #     elif not up_on and down_on:
        #         state = STATE_CLOSED
        # else:
        return (STATE_OPENING if up_on and not down_on else STATE_CLOSING
                if not up_on and down_on else STATE_OPEN)

    # @property
    # def should_stop(self):
    #     """It is watching if the time to stop evaluate or not"""

    #     # stop the shutter
    #     result = True

    #     if self._timer.is_running:
    #         # timer is still working
    #         self._timer.update_tick()

    #         # when the timer reach of counting, then stop it
    #         # otherwise return false
    #         if self._timer.elapsed_time < 0:
    #             self._timer.stop()
    #         else:
    #             result = False

    #     return result

    @property
    def supported_features(self):
        """Definition what the devices supports."""
        return SUPPORT_OPEN \
            | SUPPORT_CLOSE \
            | SUPPORT_SET_POSITION \
            | SUPPORT_STOP \
            | SUPPORT_OPEN_TILT \
            | SUPPORT_CLOSE_TILT \
            | SUPPORT_STOP_TILT

    # @property
    # def current_position(self):
    #     """Current position of the shutter."""
    #     # It is calculated from the time to close the shutter,
    #     # defined with pull up or pull down fnc called.
    #     # 0 - fully closed - MIN_RANGE
    #     # 100 - fully opened - MAX_RANGE
    #     state = self.state

    #     position = self.__last_position

    #     # this is a situation when is not set the timer to count
    #     if self.__time_to_stop == 0:
    #         position = MAX_RANGE if state is STATE_CLOSING else MIN_RANGE
    #     else:
    #         percent = 0
    #         # calculate the position based on time to stop and current
    #         # tick of the timer
    #         if self._timer.tick is not None:
    #             tick = int(self._timer.tick)
    #             percent = percent if tick == 0 else (
    #                 tick / self.__time_to_stop) * 100

    #             percent = int(MAX_RANGE if percent > MAX_RANGE else percent)
    #         # when the timer stops and last position is the same as one
    #         # of the range side then return the last position
    #         elif self._timer.tick is None and \
    #                 (self.__last_position == MIN_RANGE \
    #                       or self.__last_position
    #                     == MAX_RANGE):
    #             return self.__last_position

    #         if state is STATE_CLOSING:
    #             position = MIN_RANGE if position < MIN_RANGE \
    #                 else MAX_RANGE - percent
    #         elif state is STATE_OPENING:
    #             position = MAX_RANGE if position > MAX_RANGE \
    #                 else MIN_RANGE + percent
    #         elif state is STATE_CLOSED:
    #             position = MIN_RANGE
    #         elif state is STATE_OPEN:
    #             position = MAX_RANGE

    #     self.__last_position = position

    #     return int(self.__last_position)

    async def pull_up(self, stop_after=None):
        """Turn up the shutter."""
        # self.__set_time_to_stop(stop_after)
        # self._timer.start(self.__time_to_stop)

        await self.__call_service(DIRECTIONS_DICT.get(ATTR_UP))

    async def pull_down(self, stop_after=None):
        """ Turn down the shutter."""
        # self.__set_time_to_stop(stop_after)
        # self._timer.start(self.__time_to_stop)

        await self.__call_service(DIRECTIONS_DICT.get(ATTR_DOWN))

    async def stop(self):
        """ Stop the shutter."""
        # if self._timer.is_running:
        #     self._timer.stop()

        await self.__call_service(DIRECTIONS_DICT.get(ATTR_STOP))

    async def __call_service(self, direction):
        """Internal call of the device write value."""

        if direction == DIRECTIONS_DICT.get(ATTR_STOP):
            await self._device.write_value(self.__set_value(
                ATTR_SWITCH_OFF, ATTR_SWITCH_OFF))
        elif direction == DIRECTIONS_DICT.get(ATTR_UP):
            await self._device.write_value(self.__set_value(
                ATTR_SWITCH_OFF, ATTR_SWITCH_ON))
        elif direction == DIRECTIONS_DICT.get(ATTR_DOWN):
            await self._device.write_value(self.__set_value(
                ATTR_SWITCH_ON, ATTR_SWITCH_OFF))

    def __set_value(self, down, up):
        """Set the value to call service."""
        return {f'{self._device.down}': down, f'{self._device.up}': up}

    # def __set_time_to_stop(self, stop_after):
    #     """Set time to stop private function."""
    #     if stop_after is not None:
    #         self.__time_to_stop = stop_after



"""Unit testing of iNels switch library."""
from pyinels.const import ATTR_SWITCH

from tests.const_test import (
    TEST_API_CLASS_NAMESPACE,
    TEST_API_ROOM_DEVICES,
    TEST_API_READ_DATA,
    TEST_HOST,
    TEST_PORT,
    TEST_RAW_SWITCH,
    TEST_VERSION
)

from pyinels.api import Api
from pyinels.device.pySwitch import pySwitch

from unittest.mock import patch
from unittest import async_case

SWITCH_ID = "ZAS_13A_Loznice"
SWITCH_NAME = "Zas loznice"

SWITCH_RETURN_OFF = {SWITCH_ID: 0}
SWITCH_RETURN_ON = {SWITCH_ID: 1}


class PySwitchTest(async_case.IsolatedAsyncioTestCase):
    """Class to test iNels switch library."""

    def setUp(self):
        """Setup all necesary instances and mocks."""
        self.patches = [
            patch(f'{TEST_API_CLASS_NAMESPACE}.ping', return_value=True),
            patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_ROOM_DEVICES}',
                  return_value=TEST_RAW_SWITCH),
            patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_READ_DATA}',
                  return_value=SWITCH_RETURN_OFF),
            patch(f'{TEST_API_CLASS_NAMESPACE}._Api__writeValues',
                  return_value=None)
        ]

        for p in self.patches:
            p.start()

        self.api = Api(TEST_HOST, TEST_PORT, TEST_VERSION)

    async def asyncSetUp(self):
        """Setup all neccessary async stuff."""
        switches = [device for device in await self.api.getRoomDevices(
            'garage') if device.type == ATTR_SWITCH]

        self.switch = await pySwitch(switches[0])

    def tearDown(self):
        """Destroy all instances and mocks."""
        self.api = None
        self.switch = None
        patch.stopall()
        self.patches = None

    async def test_state(self):
        """Test the state of the pySwitch."""
        s = self.switch

        await s.update()
        # the switch at the beggining should be turned off
        self.assertFalse(s.state)

    def test_unique_id_and_name_presented(self):
        """Test when the unique id is presented."""
        s = self.switch

        self.assertIsNotNone(s.unique_id)
        self.assertIsNotNone(s.name)

        self.assertEqual(s.unique_id, SWITCH_ID)
        self.assertEqual(s.name, SWITCH_NAME)

    async def test_turn_on(self):
        """Test turn on the switch."""
        s = self.switch

        await s.update()
        self.assertFalse(s.state)

        with patch.object(self.api, TEST_API_READ_DATA,
                          return_value=SWITCH_RETURN_ON):
            await s.turn_on()

            await s.update()
            self.assertTrue(s.state)

    async def test_turn_off(self):
        """Test turn off the switch."""
        s = self.switch

        with patch.object(self.api, TEST_API_READ_DATA,
                          return_value=SWITCH_RETURN_ON):
            await s.turn_on()

            await s.update()
            self.assertTrue(s.state)

        await s.turn_off()

        await s.update()
        self.assertFalse(s.state)



"""Library specified for iNels BUS."""
import logging

from dataclasses import dataclass

_LOGGER = logging.getLogger(__name__)


@dataclass
class ApiException(Exception):
    """Base iNels BUS exception."""
    code: str
    message: str
    trace: Exception = None


@dataclass
class ApiConnectionException(ApiException):
    """Connection exception class."""


@dataclass
class ApiDataTypeException(ApiException):
    """Bad type exception."""


@dataclass
class ApiClassTypeException(ApiException):
    """Bad Class type exception."""



"""Class Api resources returned from the iNels BUS."""
import logging
import asyncio
import concurrent.futures

from pyinels.exception import ApiClassTypeException

from pyinels.const import (
    ATTR_DOWN,
    ATTR_GROUP,
    ATTR_ID,
    ATTR_READ_ONLY,
    ATTR_RELE,
    ATTR_TEMP,
    ATTR_TEMP_SET,
    ATTR_TITLE,
    ATTR_TYPE,
    ATTR_UP,
    INELS_BUS_ATTR_DICT
)

_LOGGER = logging.getLogger(__name__)


class ApiResource:
    """Object returned from the iNels BUS."""

    @property
    def id(self):
        """Id of the object."""
        if INELS_BUS_ATTR_DICT.get(ATTR_ID) not in self.__json:
            return None
        return self.__json[INELS_BUS_ATTR_DICT.get(ATTR_ID)]

    @property
    def title(self):
        """Name of the object."""
        return self.__json[INELS_BUS_ATTR_DICT.get(ATTR_TITLE)]

    @property
    def type(self):
        """Type of the object."""
        return self.__json[INELS_BUS_ATTR_DICT.get(ATTR_TYPE)]

    @property
    def temperature(self):
        """Temperature of the object."""
        if INELS_BUS_ATTR_DICT.get(ATTR_TEMP) not in self.__json:
            return None
        return self.__json[INELS_BUS_ATTR_DICT.get(ATTR_TEMP)]

    @property
    def temperature_set(self):
        """Temperature of the object."""
        if INELS_BUS_ATTR_DICT.get(ATTR_TEMP_SET) not in self.__json:
            return None
        return self.__json[INELS_BUS_ATTR_DICT.get(ATTR_TEMP_SET)]

    @property
    def rele(self):
        """Rele of the object."""
        if INELS_BUS_ATTR_DICT.get(ATTR_RELE) not in self.__json:
            return None
        return self.__json[INELS_BUS_ATTR_DICT.get(ATTR_RELE)]

    @property
    def read_only(self):
        """Read only of the object."""
        if INELS_BUS_ATTR_DICT.get(ATTR_READ_ONLY) not in self.__json:
            return None
        return self.__json[INELS_BUS_ATTR_DICT.get(ATTR_READ_ONLY)]

    @property
    def down(self):
        """Shutter down of the object."""
        if INELS_BUS_ATTR_DICT.get(ATTR_DOWN) not in self.__json:
            return None
        return self.__json[INELS_BUS_ATTR_DICT.get(ATTR_DOWN)]

    @property
    def up(self):
        """Shutter up of the object."""
        if INELS_BUS_ATTR_DICT.get(ATTR_UP) not in self.__json:
            return None
        return self.__json[INELS_BUS_ATTR_DICT.get(ATTR_UP)]

    @property
    def group(self):
        """Group from where the object is loaded."""
        if INELS_BUS_ATTR_DICT.get(ATTR_GROUP) not in self.__json:
            return None
        return self.__json[INELS_BUS_ATTR_DICT.get(ATTR_GROUP)]

    @property
    def value(self):
        """Value of the returned object."""
        if hasattr(self, '_ApiResource__value'):
            return self.__value

        return None

    def set_value(self, value):
        self.__value = value

    async def write_value(self, value):
        """Set value to the device."""

        if isinstance(value, int) or isinstance(value, float):
            value = {f'{self.id}': f'{value}'}

        await asyncio.create_task(self.__api.write(self, value))
        self.set_value(value)

    def __init__(self, json, api):
        """Initializer of the Api resource."""
        self.__json = json
        self.__api = api
        self.__value = None
        self.pool = concurrent.futures.ThreadPoolExecutor()

    async def observe(self):
        """Read the current value of the device."""
        try:
            raw = None

            # shutter in action
            if self.up is not None and \
                    self.down is not None:

                raw = await asyncio.create_task(
                    self.__api.read([self.up, self.down]))
            else:
                raw = await asyncio.create_task(
                    self.__api.read([self.id]))

            self.set_value(raw)

            return self.value
        except ApiClassTypeException as ex:
            self.set_value(None)
            raise ex
        except Exception as ex:
            raise ex

        return None

    @property
    def is_available(self):
        """Device availability property."""
        value = None
        # first test when device has any value
        if (hasattr(self, '_ApiResource__value')
                and self._ApiResource__value is not None):
            return True
        else:
            # if not then try to observer
            value = self.pool.submit(asyncio.run, self.observe()).result()
        # when the result is None then the device is not available
        return False if value[self.id] is None else True



"""Unit testing of iNels shutter library."""

# import asyncio

from unittest.mock import patch
from unittest import async_case

from pyinels.api import Api
from pyinels.device.pyShutter import pyShutter

from pyinels.const import (
    ATTR_SHUTTER,
    # STATE_CLOSED,
    STATE_CLOSING,
    STATE_OPEN,
    STATE_OPENING
)

from tests.const_test import (
    TEST_API_CLASS_NAMESPACE,
    TEST_API_ROOM_DEVICES,
    TEST_API_READ_DATA,
    TEST_HOST,
    TEST_PORT,
    TEST_RAW_SHUTTER,
    TEST_VERSION,
    TEST_SHUTTER_UP,
    TEST_SHUTTER_DOWN,
    TEST_SHUTTER_ID,
    TEST_SHUTTER_NAME,
    TEST_RETURN_RESOURCE_SHUTTER,
    TEST_RETURN_RESOURCE_SHUTTER_UP,
    TEST_RETURN_RESOURCE_SHUTTER_DOWN
)


SHUTTER_UP_ON = {TEST_SHUTTER_UP: 1}
SHUTTER_UP_OFF = {TEST_SHUTTER_UP: 0}

SHUTTER_DOWN_ON = {TEST_SHUTTER_DOWN: 1}
SHUTTER_DOWN_OFF = {TEST_SHUTTER_DOWN: 0}


class PyShutterTest(async_case.IsolatedAsyncioTestCase):
    """Class to test iNels shutter library."""

    def setUp(self):
        """Setup all necesary instances and mocks."""
        self.patches = [
            patch(f'{TEST_API_CLASS_NAMESPACE}.ping', return_value=True),
            patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_ROOM_DEVICES}',
                  return_value=TEST_RAW_SHUTTER),
            patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_READ_DATA}',
                  return_value=TEST_RETURN_RESOURCE_SHUTTER),
            patch(f'{TEST_API_CLASS_NAMESPACE}._Api__writeValues',
                  return_value=None)
        ]

        for item in self.patches:
            item.start()

        self.api = Api(TEST_HOST, TEST_PORT, TEST_VERSION)

    async def asyncSetUp(self):
        """Setup all neccessary async stuff."""
        shutters = [device for device in await self.api.getRoomDevices(
            'garage') if device.type == ATTR_SHUTTER]

        self.shutter = await pyShutter(shutters[0])

    def tearDown(self):
        """Destroy all instances and mocks."""
        self.api = None
        self.shutter = None
        patch.stopall()
        self.patches = None

    async def test_state_after_initialization(self):
        """Test the state of the pyShutter."""
        shutt = self.shutter

        await shutt.update()
        # the shutter at the beggining should be none. There will be
        # calibration
        self.assertIs(shutt.state, STATE_OPEN)

    def test_unique_id_and_name_presented(self):
        """Test when the unique id is presented."""
        shutt = self.shutter

        self.assertIsNotNone(shutt.unique_id)
        self.assertIsNotNone(shutt.name)

        self.assertEqual(shutt.unique_id, TEST_SHUTTER_ID)
        self.assertEqual(shutt.name, TEST_SHUTTER_NAME)

    async def test_full_opening(self):
        """Test full open shutter."""
        shutt = self.shutter

        await shutt.pull_up()

        with patch.object(self.api, TEST_API_READ_DATA,
                          return_value=TEST_RETURN_RESOURCE_SHUTTER_UP):

            # self.assertFalse(shutt.should_stop)
            self.assertEqual(shutt.state, STATE_OPENING)

            # asyncio.sleep(1)
            # self.assertFalse(shutt.should_stop)
            # self.assertEqual(shutt.state, STATE_OPENING)

            # asyncio.sleep(1)
            # self.assertTrue(shutt.should_stop)
            # self.assertEqual(shutt.state, STATE_OPEN)

    async def test_full_close(self):
        """Test to full close the shutter."""
        shutt = self.shutter

        await shutt.pull_down()

        with patch.object(self.api, TEST_API_READ_DATA,
                          return_value=TEST_RETURN_RESOURCE_SHUTTER_DOWN):

            # self.assertFalse(shutt.should_stop)
            self.assertEqual(shutt.state, STATE_CLOSING)

            # asyncio.sleep(1)
            # self.assertFalse(shutt.should_stop)
            # self.assertEqual(shutt.state, STATE_CLOSING)

            # asyncio.sleep(1)
            # self.assertTrue(shutt.should_stop)
            # self.assertEqual(shutt.state, STATE_CLOSED)

    # async def test_current_position_closing(self):
    #     """Testing current poistion of the shutter for closing."""
    #     shutt = self.shutter

    #     # initial state is fully opened so it means 100%
    #     await shutt.pull_down(3)
    #     asyncio.sleep(1)

    #     with patch.object(self.api, TEST_API_READ_DATA,
    #                       return_value=TEST_RETURN_RESOURCE_SHUTTER_DOWN):
    #         self.assertIs(shutt.current_position, 67)

    #     asyncio.sleep(1)

    #     with patch.object(self.api, TEST_API_READ_DATA,
    #                       return_value=TEST_RETURN_RESOURCE_SHUTTER_DOWN):
    #         self.assertIs(shutt.current_position, 34)

    #     asyncio.sleep(1)

    #     with patch.object(self.api, TEST_API_READ_DATA,
    #                       return_value=TEST_RETURN_RESOURCE_SHUTTER_DOWN):
    #         self.assertIs(shutt.current_position, 0)

    # async def test_current_position_opening(self):
    #     """Test current position of the sutter for opening."""

    #     shutt = self.shutter

    #     # initial state is fully opened so it means 100%
    #     await shutt.pull_down(0)
    #     asyncio.sleep(1)

    #     with patch.object(self.api, TEST_API_READ_DATA,
    #                       return_value=TEST_RETURN_RESOURCE_SHUTTER_DOWN):
    #         self.assertIs(shutt.current_position, 0)

    #     await shutt.pull_up(3)
    #     asyncio.sleep(1)

    #     with patch.object(self.api, TEST_API_READ_DATA,
    #                       return_value=TEST_RETURN_RESOURCE_SHUTTER_DOWN):
    #         self.assertIs(shutt.current_position, 33)

    #     asyncio.sleep(1)

    #     with patch.object(self.api, TEST_API_READ_DATA,
    #                       return_value=TEST_RETURN_RESOURCE_SHUTTER_DOWN):
    #         self.assertIs(shutt.current_position, 66)

    #     asyncio.sleep(1)

    #     with patch.object(self.api, TEST_API_READ_DATA,
    #                       return_value=TEST_RETURN_RESOURCE_SHUTTER_DOWN):
    #         self.assertIs(shutt.current_position, 100)



"""Const of iNels BUS tests."""

TEST_API_READ_DATA = "_Api__readDeviceData"
TEST_API_ROOM_DEVICES = "getRoomDevicesRaw"

TEST_API_NAMESPACE = "pyinels.api"
TEST_API_CLASS_NAMESPACE = "pyinels.api.Api"

TEST_DATA_THERM = {
    'title': 'Valve in bedroom',
    'temp': 'Bedroom_temerature',
    'rele': 'DIGITALGROUP_0001',
    'read_only': 'no',
    'temp_set': 'Bedroom_stateth'
}

TEST_DATA_LIGHT = {
    'title': 'Main light',
    'id': 'SV_12_Garage',
    'read_only': 'no'
}

TEST_DATA_SWITCH = {
    'title': 'Kitchen kettel switch',
    'id': 'KITCHEN_KETTLE_SWITCH',
    'read_only': 'no'
}

TEST_DATA_SHUTTER = {
    'title': 'Kitchen shuuter',
    'up': 'KITCHEN_SHUUTER_1_UP',
    'down': 'KITCHEN_SHUUTER_1_DOWN',
    'read_only': 'no'
}

TEST_DATA_DOOR = {
    'title': 'Garage door',
    'id': 'GARAGE_DOOR',
    'read_only': 'no'
}

TEST_DATA_HEATING = {
    'title': 'Holiday',
    'id': 'HOLIDAY_HEATING',
    'read_only': 'no'
}

TEST_HOST = "http://localhost"

TEST_INELS_DEVICE_NAMESPACE = "pyinels.device.Device"

TEST_PORT = "1000"

TEST_RESOURCE_SWITCH = {
    'column': '0',
    'group': 'basement',
    'inels': 'ZA_14_basement',
    'name': 'Office switch',
    'read_only': 'no',
    'row': '1',
    'type': 'switch'
}

TEST_SHUTTER_UP = "ZAL__ahor"
TEST_SHUTTER_DOWN = "Z_Obk_do"
TEST_SHUTTER_ID = f"{TEST_SHUTTER_UP}_{TEST_SHUTTER_DOWN}"
TEST_SHUTTER_NAME = "Zal OB"

TEST_RETURN_RESOURCE_SHUTTER = {
    f'{TEST_SHUTTER_DOWN}': 0,
    f'{TEST_SHUTTER_UP}': 0
}

TEST_RETURN_RESOURCE_SHUTTER_UP = {
    f'{TEST_SHUTTER_DOWN}': 0,
    f'{TEST_SHUTTER_UP}': 1
}

TEST_RETURN_RESOURCE_SHUTTER_DOWN = {
    f'{TEST_SHUTTER_DOWN}': 1,
    f'{TEST_SHUTTER_UP}': 0
}

TEST_RAW_LIGHT = """
lights:
name="Main light" column="0" inels="SV_12_Garage" read_only="no" row="0"
"""

TEST_RAW_SWITCH = """
on_off:
name="Zas loznice" column="3" inels="ZAS_13A_Loznice" read_only="no" row="0"
"""

TEST_RAW_GARAGE_DOOR = """
garage:
name="Vrata" column="0" inels="Vrata_Garaz" read_only="no" row="1"
"""

TEST_RAW_SHUTTER = """
shutters:
name="Zal OB" column="0" down="Z_Obk_do" read_only="no" up="ZAL__ahor" row="3"
"""

TEST_RAW_DUPLICIT_DEVICES = """on_off:
name="Zas OB" column="0" inels="ZAS_1A_Obyvak" read_only="no" row="2"
name="Zas POK" column="1" inels="ZAS_1B_Pokoj_dole" read_only="no" row="2"
name="Zas POK" column="1" inels="ZAS_1B_Pokoj_dole" read_only="no" row="2"
thermals:
name="Obyvak" inels="Koupelna_dole_StateTH" placement="indoor"
heating:
name="DOVOLENA" column="0" inels="DIGITALGROUP_0001" read_only="no" row="5"
lights:
name="Obyvak" column="0" inels="SV_1_Obyvak_strop" read_only="no" row="0"
name="Stul" column="1" inels="SV_2_Jidelni_stul" read_only="no" row="0"
name="Kuchyn" column="2" inels="SV_3_Kuchyn" read_only="no" row="0"
name="Linka" column="3" inels="SV_4_Kuchyn_linka" read_only="no" row="0"
name="Schody" column="0" inels="SV_15_Schodiste" read_only="no" row="1"
name="Vstup" column="1" inels="SV_10_Chodba_dole" read_only="no" row="1"
name="Pokoj" column="2" inels="SV_7_Pokoj_dole" read_only="no" row="1"
name="Koupelna" column="3" inels="SV_9_Koupelna_dole" read_only="no" row="1"
name="Technicka" column="4" inels="SV_11_Technicka_1." read_only="no" row="1"
name="Pod schody" column="5" inels="SV_14_Kumbal_rozvc" read_only="no" row="1"
shutters:
name="Zal OB" column="0" down="Z_Obk_do" read_only="no" up="ZAL__ahor" row="3"
name="Rol KRB" column="1" down="ROL_Jillu" read_only="no" up="ROJidru" row="3"
name="Rol KUCH" column="2" down="RL_Kc_do" read_only="no" up="ROuchyu" row="3"
name="Rol POK" column="3" down="R_Po_dou" read_only="no" up="ROnahoru" row="3"
heat-control:
name="Obyvak" read_only="no" therm="O" column="0" stateth="O" rele="D" row="4"
name="Pokoj" read_only="no" therm="P" column="1" stateth="P" rele="D" row="4"
name="Koupel" read_only="no" therm="K" column="2" stateth="K" rele="D" row="4"
name="Zadveri" read_only="no" therm="V" column="3" stateth="V" rele="D" row="4"
name="Obyvak" read_only="no" therm="O" column="0" stateth="O" rele="D" row="4"
name="Pokoj" read_only="no" therm="P" column="1" stateth="P" rele="D" row="4"
name="Koupel" read_only="no" therm="K" column="2" stateth="K" rele="D" row="4"
name="Zadveri" read_only="no" therm="V" column="3" stateth="V" rele="D" row="4"
lights:
name="Fasada" column="0" inels="SV_6_Terasa_Fasada" read_only="no" row="0"
name="Garaz zadni vch" column="1" inels="SV_G_Za_V" read_only="no" row="0"
name="Terasa" column="2" inels="SV_Terasa_Zem" read_only="no" row="0"
name="Vchod a garaz" column="3" inels="SV_10_Ven_Garaz" read_only="no" row="0"
shutters:
name="Markyza" column="0" down="Marky_d" read_only="no" up="Marky_n" row="1"
lights:
name="Loznice" column="0" inels="SV_19_Loznice" read_only="no" row="0"
name="Satna" column="1" inels="SV_20_Loznice_satna" read_only="no" row="0"
name="Zachod" column="0" inels="SV_21_WC_patro" read_only="no" row="1"
name="Chodba" column="1" inels="SV_18_Chodba_2.NP" read_only="no" row="1"
name="Schodiste" column="2" inels="SV_15_Schodiste" read_only="no" row="1"
name="Puda" column="3" inels="SV_Puda" read_only="no" row="1"
name="Koupelna" column="0" inels="SV_22_Koupelna_nahore" read_only="no" row="2"
name="Umyvadlo" column="1" inels="SV_22_Umyvadlo" read_only="no" row="2"
name="Pokoj V." column="0" inels="SV_17_Pokoj_levy" read_only="no" row="3"
on_off:
name="Zas loznice" column="3" inels="ZAS_13A_Loznice" read_only="no" row="0"
name="Zas pok V." column="2" inels="ZAS_12B_Pokoj_levy" read_only="no" row="3"
name="Zas pok Z." column="1" inels="ZAS_12A_Pokoj_pravy" read_only="no" row="4"
heat-control:
name="Lozni" read_only="no" therm="Lo" column="2" stateth="LH" rele="D" row="0"
name="Koupa" read_only="no" therm="Ko" column="2" stateth="KH" rele="D" row="2"
name="Po V." read_only="no" therm="Pl" column="1" stateth="PT" rele="D" row="3"
name="Po Z." read_only="no" therm="Pp" column="0" stateth="PH" rele="D" row="4"
lights:
name="Hl. světlo" column="0" inels="SV_12_Garaz" read_only="no" row="0"
name="Půda" column="1" inels="SV_Puda_Garaz" read_only="no" row="0"
garage:
name="Vrata" column="0" inels="Vrata_Garaz" read_only="no" row="1"
on_off:
name="VZT ZAP/VYP" column="0" inels="VZT_Zapnout" read_only="no" row="0"
name="VZT ST 2" column="1" inels="VZT_rychlost_2" read_only="no" row="0"
name="VZT ST 3" column="2" inels="VZT_Rychlost_3" read_only="no" row="0"
"""


TEST_ROOMS = ['Basement', 'First floor', 'Front yard',
              'Back yard', 'Attic', 'Garrage']

TEST_VERSION = "CU3"



"""Inels door class for iNels BUS."""
from time import sleep
from pyinels.device.pyBase import pyBase

from pyinels.const import (
    ATTR_SWITCH_ON,
    ATTR_SWITCH_OFF
)

#############################################################################
# Note!
# This class is specific. turn_off and turn_on do the same job and the state
# and the update method does not have a sense with this device
##############################################################################


class pyDoor(pyBase):
    """Inels class based on InelsDevice."""

    async def __init__(self, device):
        """Initialize pyDoor class."""
        await super().__init__(device)

    async def turn_off(self,):
        """Trigger impoulse button. Write 1 to device and then 0 back."""
        await self._device.write_value(ATTR_SWITCH_ON)
        sleep(2)  # delay to be sure that the pulse was sent
        await self._device.write_value(ATTR_SWITCH_OFF)

    async def turn_on(self,):
        """Same as turn off."""
        await self.turn_off()

    async def update(self):
        """Does not have any sens due to the pulse behavior."""
        return 0

    @property
    def state(self):
        """This property does not have any sense due to the pulse behavior."""
        return False



"""Common device class for iNels BUS."""
from pyinels.api.resources import ApiResource


class Device(ApiResource):
    """Device class."""

    def __init__(self, entity, api):
        """Initialize device class."""
        super().__init__(entity, api)

    def __repr__(self):
        """Object representation."""
        return "<{} - {} type({})>".format(self.id, self.title, self.type)



"""Unit testing of iNels light library."""
from tests.const_test import (
    TEST_API_CLASS_NAMESPACE,
    TEST_API_READ_DATA,
    TEST_API_ROOM_DEVICES,
    TEST_HOST,
    TEST_PORT,
    TEST_RAW_LIGHT,
    TEST_VERSION
)

from pyinels.const import (
    RANGE_BRIGHTNESS,
    ATTR_LIGHT
)

from pyinels.api import Api
from pyinels.device.pyLight import pyLight

from unittest.mock import patch
from unittest import async_case

MIN_RANGE = RANGE_BRIGHTNESS[0]
MAX_RANGE = RANGE_BRIGHTNESS[1]

LIGHT_ID = 'SV_12_Garage'
LIGHT_NAME = 'Main light'

LIGHT_RETURN_OFF = {LIGHT_ID: 0}
LIGHT_RETURN_ON = {LIGHT_ID: 1}

LIGHT_RETURN_DIMMABLE_OFF = {LIGHT_ID: MIN_RANGE}
LIGHT_RETURN_DIMMABLE_ON = {LIGHT_ID: MAX_RANGE}
LIGHT_RETURN_DIMMABLE_50 = {LIGHT_ID: 50}


class PyLightTest(async_case.IsolatedAsyncioTestCase):
    """Class to test iNels light library."""

    def setUp(self):
        """Setup all necesary instances and mocks."""
        self.patches = [
            patch(f'{TEST_API_CLASS_NAMESPACE}.ping', return_value=True),
            patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_ROOM_DEVICES}',
                  return_value=TEST_RAW_LIGHT),
            patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_READ_DATA}',
                  return_value=LIGHT_RETURN_OFF),
            patch(f'{TEST_API_CLASS_NAMESPACE}._Api__writeValues',
                  return_value=None)
        ]

        for p in self.patches:
            p.start()

        self.api = Api(TEST_HOST, TEST_PORT, TEST_VERSION)

    async def asyncSetUp(self):
        """Setup all neccessary async stuff."""
        lights = [device for device in await self.api.getRoomDevices(
            'garage') if device.type == ATTR_LIGHT]

        self.light = await pyLight(lights[0])

    def tearDown(self):
        """Destroy all instances and mocks."""
        self.api = None
        self.light = None
        patch.stopall()
        self.patches = None

    def test_title_and_id(self):
        """Test the name of the entity."""
        lg = self.light

        self.assertIsNotNone(lg.name)
        self.assertEqual(lg.name, LIGHT_NAME)

        self.assertIsNotNone(lg.unique_id)
        self.assertEqual(lg.unique_id, LIGHT_ID)

    async def test_state(self):
        """Test the state of the pyLight."""
        lg = self.light

        await lg.update()
        # the light at the beggining should be turned off
        self.assertFalse(lg.state)

    def test_unique_id_and_name_presented(self):
        """Test when the unique id is presented."""
        lg = self.light

        self.assertIsNotNone(lg.unique_id)
        self.assertIsNotNone(lg.name)

    async def test_turn_on(self):
        """Test turn on the light."""
        lg = self.light

        await lg.update()
        self.assertFalse(lg.state)

        with patch.object(self.api, TEST_API_READ_DATA,
                          return_value=LIGHT_RETURN_ON):
            await lg.turn_on()

            await lg.update()
            self.assertTrue(lg.state)

        await lg.turn_off()

        await lg.update()
        self.assertFalse(lg.state)

    async def test_turn_off(self):
        """Test turn off the light."""
        lg = self.light

        with patch.object(self.api, TEST_API_READ_DATA,
                          return_value=LIGHT_RETURN_ON):
            await lg.turn_on()

            await lg.update()
            self.assertTrue(lg.state)

        await lg.turn_off()

        await lg.update()
        self.assertFalse(lg.state)


class PyLightDimmableTest(async_case.IsolatedAsyncioTestCase):
    """Class to test iNels light library."""

    def setUp(self):
        """Setup all necesary instances and mocks."""
        self.patches = [
            patch(f'{TEST_API_CLASS_NAMESPACE}.ping', return_value=True),
            patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_ROOM_DEVICES}',
                  return_value=TEST_RAW_LIGHT),
            patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_READ_DATA}',
                  return_value=LIGHT_RETURN_DIMMABLE_OFF),
            patch(f'{TEST_API_CLASS_NAMESPACE}._Api__writeValues',
                  return_value=None)
        ]

        for p in self.patches:
            p.start()

        self.api = Api(TEST_HOST, TEST_PORT, TEST_VERSION)

    async def asyncSetUp(self):
        """Setup all neccessary async stuff."""
        lights = [device for device in await self.api.getRoomDevices(
            'garage') if device.type == ATTR_LIGHT]

        self.light = await pyLight(lights[0])

    def tearDown(self):
        """Destroy all instances and mocks."""
        self.api = None
        self.light = None
        patch.stopall()
        self.patches = None

    async def test_turn_on_with_brightness_option(self):
        """Test the light to turn on when the brightness exists."""

        lg = self.light

        await lg.update()
        self.assertFalse(lg.state)
        self.assertTrue(lg.has_brightness)

        with patch.object(self.api, TEST_API_READ_DATA,
                          return_value=LIGHT_RETURN_DIMMABLE_ON):
            await lg.turn_on()

            await lg.update()
            self.assertTrue(lg.state)

    async def test_set_brightness_to_50_percent(self):
        """Test the brightnes to some value."""

        lg = self.light

        await lg.update()
        self.assertFalse(lg.state)
        self.assertTrue(lg.has_brightness)

        with patch.object(self.api, TEST_API_READ_DATA,
                          return_value=LIGHT_RETURN_DIMMABLE_50):

            await lg.set_brightness(50)

            self.assertEqual(lg.brightness(), 50)
            self.assertTrue(lg.state)

    @patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_READ_DATA}')
    async def test_turn_off_with_brightness_option(self, mock_room_devices):
        """Test the light to turn off when the brightness exsists."""
        mock_room_devices.return_value = LIGHT_RETURN_DIMMABLE_ON

        lg = self.light

        self.assertTrue(lg.has_brightness)

        await lg.update()
        self.assertTrue(lg.state)

        with patch.object(self.api, TEST_API_READ_DATA,
                          return_value=LIGHT_RETURN_DIMMABLE_OFF):
            await lg.turn_off()
            self.assertTrue(lg.has_brightness)

            await lg.update()
            self.assertFalse(lg.state)
